<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    xmlns="http://docbook.org/ns/docbook" xmlns:db="http://docbook.org/ns/docbook">
    
    <xsl:param name="generateIdPrefix">__sdwordm4342cs__</xsl:param>
    <xsl:output indent="no"/>
    <xsl:param name="lastAnswerId" select="document('lastAnswerId.xml',/)/lastAnswer/@id" />
    
    
    <xsl:key name="internRef" match="//*[@linkend]" use="@linkend"/>
    
    <xsl:template match="@* | node()">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="*" mode="hdmIdGenerate">
        <xsl:choose>
            <xsl:when test="@xml:id">
                <xsl:value-of select="@xml:id"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$generateIdPrefix"/>
                <xsl:value-of select="generate-id()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="db:figure[db:mediaobject/db:imageobject]/db:title">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
        <para>
            <xsl:if test="substring-after(parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref, '.') = 'fig'">
                <uri>
                    <xsl:attribute name="xlink:href">
                        <xsl:call-template name="hdm.get.leftmost.of.seperator">
                            <xsl:with-param name="path" select="ancestor::*/@xml:base"/>
                            <xsl:with-param name="seperator" select="'/'"/>
                        </xsl:call-template>                  
                        <xsl:text>/</xsl:text>
                        <xsl:call-template name="hdm.get.leftmost.of.seperator">
                            <xsl:with-param name="path" 
                                select="parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref"/>
                            <xsl:with-param name="seperator" select="'.'"/>
                        </xsl:call-template>                  
                        <xsl:text>.pdf</xsl:text>
                    </xsl:attribute>
                    <xsl:text>(Animated PDF Version)</xsl:text>
                </uri>
            </xsl:if>
        </para>
    </xsl:template>
   
   <!-- 
    <xsl:template match="db:answer">
        <db:answer>
            <xsl:variable name="proximaParentId" select="ancestor::*[@xml:id and position()=1]/@xml:id"/>
            <xsl:choose>
                <xsl:when test="following::*[@xml:id=$lastAnswerId] or ancestor::*[@xml:id=$lastAnswerId]">
                        <xsl:apply-templates select="@* | node()"/>
                </xsl:when>
                <xsl:otherwise>
                    <para>Answer id=<xsl:value-of select="ancestor::*[@xml:id][1]/@xml:id"/> not yet published.</para>
                </xsl:otherwise>
            </xsl:choose>
        </db:answer>
    </xsl:template>
   --> 
    <xsl:template match="db:part">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
            <appendix>
                <title>
                    <xsl:text>List of exercise sets in </xsl:text>
                    <xsl:value-of select="db:title" />
                </title>
                
                <table border="1" xml:id="{@xml:id}solutionsExercise">
                    <caption>Index of excersises</caption>
                    <col width="60%" />
                    <col width="40%" />
                    <tr>
                        <th>Exercise title / page</th>
                        <th>Status of completion</th>
                    </tr>
                    <xsl:for-each select=".//db:qandaset">
                        <tr>
                            <td> 
                                <link linkend="{@xml:id}"> <!--  xrefstyle="template:%n, %t" -->
                                    <xsl:choose>
                                        <xsl:when test="db:title">
                                            <xsl:copy-of select="db:title/node()"/>
                                        </xsl:when>
                                        <xsl:otherwise>
                                             <xsl:copy-of select="preceding-sibling::db:title[1]/node()"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </link>
                            </td>
                            <td/>
                        </tr>
                    </xsl:for-each>
                </table>
            </appendix>
        </xsl:copy>        
    </xsl:template>
    
    <xsl:template name="hdm.get.leftmost.of.seperator">
        <xsl:param name="path"/>
        <xsl:param name="seperator"/>
        <xsl:param name="start">1</xsl:param>
        <xsl:if test="contains($path, $seperator )">
            <xsl:if test="$start='0'">
                <xsl:value-of select="$seperator"/>
            </xsl:if>
            <xsl:value-of select="substring-before($path, $seperator)"/>
            <xsl:call-template name="hdm.get.leftmost.of.seperator">
                <xsl:with-param name="path" select="substring-after($path, $seperator)"/>
                <xsl:with-param name="seperator" select="$seperator"/>
                <xsl:with-param name="start">0</xsl:with-param>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>    
    
    <!-- Excluding non-referenced <bibliography> and <glossary> elements -->
 
    <xsl:template match="db:bibliography/db:biblioentry[not(key('internRef', @xml:id))]">
        <xsl:comment>
             <xsl:text>Excluding non referenced bibliography entry '</xsl:text>
             <xsl:value-of select="db:abbrev"/>
             <xsl:text>', @xml:id='</xsl:text>
            <xsl:value-of select="@xml:id"/>
            <xsl:text>'</xsl:text>
        </xsl:comment>
    </xsl:template>
    
    <xsl:template match="db:glossary/db:glosslist//db:glossentry[not(key('internRef', @xml:id)) and not(key('internRef', .//*/@xml:id))]">
        <xsl:comment>
             <xsl:text>Excluding non referenced glossary entry '</xsl:text>
             <xsl:value-of select="db:glossterm"/>
             <xsl:text>', @xml:id='</xsl:text>
            <xsl:value-of select="@xml:id"/>
            <xsl:text>'</xsl:text>
         </xsl:comment>
    </xsl:template>
    
    <!-- 
    <xsl:template match="db:bibliography">
        <db:bibliography>
            <xsl:apply-templates select="*[not(name(.) = 'biblioentry')]|db:biblioentry[key('internRef', @xml:id)]"/>
        </db:bibliography>
    </xsl:template>
 

<xsl:template match="db:glossary/db:glosslist">
        <db:glosslist>
            <xsl:apply-templates select="*[not(name(.) = 'glossentry')]|db:glossentry[key('internRef', @xml:id)]"/>
        </db:glosslist>
    </xsl:template>
     -->
    
    <!-- make annotations -->
    
    <xsl:template match="db:annotation[@role='make']">
        <xsl:apply-templates select="*" mode="make"/>
    </xsl:template>
    
    <xsl:template match="db:para[@role='eclipse']" mode='make'>
        
        <xsl:variable name="baseDir">
            <xsl:text>P/</xsl:text>
            <xsl:value-of select="."/>
            <xsl:text>/target</xsl:text>
        </xsl:variable>
        
        <itemizedlist>
            <listitem>
                <para>
                    <xsl:text>Sourcecode available as importable Eclipse Maven project </xsl:text>
                    <link xlink:href="{$baseDir}/project.zip">zip archive</link>
                    <xsl:text>.</xsl:text>
                </para>
            </listitem>
            
            <listitem>
                <para>Online browsing of <link xlink:href="{$baseDir}/site/apidocs/allclasses-noframe.html">API
                    and implementation</link>.</para>
            </listitem>
        </itemizedlist>
    </xsl:template>
    
    <xsl:template match="*" mode="make">
        <para>
            <xsl:text>No template of mode='make' defined for element'</xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text>'</xsl:text>
        </para>
        
    </xsl:template>
    
</xsl:stylesheet>
